import sqlite3
import random
import unittest
import pandas as pd
from bd.bdmemoria import *
from estaticos.esquema_bd import *


class TestBaseDeDatosTablasOperaciones(unittest.TestCase):

    def setUp(self):
        self.bd = sqlite3.connect(":memory:")

    def test_crear_bd_en_memoria_devuelve_conexion(self):
        """
        Verifica que la funcion para crear la base de datos en memoria devuelva un objeto proveniente
        de SQLite3 de tipo Connection.
        """
        conexion = crear_bd_en_memoria()

        self.assertIsInstance(conexion, sqlite3.Connection)

    def test_crear_tabla_en_bd_memoria_campos_texto(self):
        """
        Verifica que se crean las tablas dadas, con los campos correctos.
        """
        campos = {
            'campo_uno': 'text',
            'campo_dos': 'text',
            'campo_tres': 'text'
        }

        crear_tabla(conexion=self.bd,
                    tabla='tabla_prueba',
                    **campos)

        columnas = pd.read_sql_query(f"SELECT * FROM tabla_prueba", self.bd).columns

        self.assertTrue('campo_uno' in columnas)
        self.assertTrue('campo_dos' in columnas)
        self.assertTrue('campo_tres' in columnas)

    def test_insertar_en_tabla(self):
        """
        Verifica que se inserto un registro en cierta tabla
        """

        campos = {
            'campo_uno': 'text',
            'campo_dos': 'text',
            'campo_tres': 'text'
        }

        crear_tabla(conexion=self.bd,
                    tabla='tabla_prueba',
                    **campos)

        registro = ['uno', 'dos', 'tres']

        insertar(self.bd, 'tabla_prueba', *registro)

        registros = pd.read_sql_query(f"SELECT * FROM tabla_prueba", self.bd).values

        self.assertIn('uno', registros)
        self.assertIn('dos', registros)
        self.assertIn('tres', registros)


class TestBaseDeDatosTablasQuerys(unittest.TestCase):

    def setUp(self):
        """
        Al iniciar los tests, creamos una base de datos en memoria y una tabla que populamos con 20 registros.
        """
        self.bd = sqlite3.connect(":memory:")

        tabla1 = {
            'campo1_tabla1': 'text',
            'campo2_tabla1': 'text',
            'campo3_tabla1': 'text',
        }

        crear_tabla(conexion=self.bd, tabla='tabla_prueba', **tabla1)

        registros = ['uno', 'dos', 'tres']

        for i in range(20):
            insertar(self.bd, 'tabla_prueba', *registros)

    def test_obtener_todos_los_registros_de_una_tabla(self):
        """
        Verifica que la funcion obtener_todos devuelva una lista con elementos Row de la libreria de SQLite3
        """
        registros = obtener_todos(self.bd, 'tabla_prueba')

        self.assertIsInstance(registros, list)

        for registro in registros:
            self.assertIsInstance(registro, sqlite3.Row)

    def test_obtener_mensajes_pendientes_tabla_existe(self):
        """
        Verifica que la funcion obtener_pendientes devuelve una lista con elementos Row.
        En este test se crea primero la tabla 'mensajes' con una populacion pequeña de registros.

        Se utiliza el diccionario esq_mensajes de esquema_bd.py para mantener el test utilizable frente a cambios
        en los esquemas.
        """
        crear_tabla(self.bd, 'mensajes', **esq_mensajes)

        # Ingresamos los registros con los mensajes como pendientes
        for i in range(5):
            insertar(self.bd, 'mensajes',
                     *[str(i) + "ABCD",  # Id del mensaje
                       "+5493415846599",  # Telefono
                       "Matias",  # Nombre agendado
                       "El mensaje jaja",  # Texto del mensaje
                       "txt",  # Clase del mensaje
                       "12:30 15/02/2021",  # Fecha y hora
                       "pendiente"])  # Estado del mensaje

        # Insertamos los registros con los mensajes como enviados
        for i in range(5, 11):
            insertar(self.bd, 'mensajes',
                     *[str(i) + "ABCD",
                       "+5493415846599",
                       "Matias",
                       "El mensaje jaja",
                       "txt",
                       "12:30 15/02/2021",
                       "enviado"])

        pendientes = obtener_pendientes(self.bd)

        self.assertIsInstance(pendientes, list)

        for pendiente in pendientes:
            self.assertIsInstance(pendiente, sqlite3.Row)
            self.assertEqual(pendiente['estado'], 'pendiente')

    def test_obtener_mensajes_pendientes_tabla_no_existe(self):
        """
        Verifica que la funcion obtener_pendientes levanta una excepcion cuando no se ha creado la tabla 'mensajes'
        previo a la ejecucion de la funcion.
        """
        with self.assertRaises(sqlite3.OperationalError):
            obtener_pendientes(self.bd)

    def test_obtener_mensajes_a_enviar_con_registros_tabla_existe(self):
        """
        Verifica que la funcion obtener_mensajes_a_enviar devuelve una lista de elementos Row,
        que su estado sea 'pendiente' y que poseen la key 'rowid' (La id unica del registro)
        """
        crear_tabla(self.bd, 'fila', **esq_fila)

        # Insertamos los registros de mensajes pendientes
        for i in range(5, 11):
            insertar(self.bd, 'fila',
                     *["+5493415846599",  # Telefono
                       "El mensaje jeje",  # Texto del mensaje
                       "pendiente",  # Estado del mensaje
                       "Matias",  # Nombre agendado
                       "12:30 15/02/2021"])  # Fecha y hora

        # Insertamos los registros de mensajes ya enviados previamente
        for i in range(5, 11):
            insertar(self.bd, 'fila',
                     *["+5493415846599",  # Telefono
                       "El mensaje jeje",  # Texto del mensaje
                       "enviado",  # Estado del mensaje
                       "Matias",  # Nombre agendado
                       "12:30 15/02/2021"])  # Fecha y hora

        pendientes = obtener_mensajes_a_enviar(self.bd)

        self.assertIsInstance(pendientes, list)

        for pendiente in pendientes:
            self.assertIn('rowid', pendiente.keys())
            self.assertTrue(pendiente['estado'] == 'pendiente')

    def test_obtener_mensajes_a_enviar_sin_registros_tabla_existe(self):
        """
        Verifica que la funcion obtener_mensajes_a_enviar devuelve una lista vacia cuando no hay
        registros pendientes en la tabla 'fila'
        """
        crear_tabla(self.bd, 'fila', **esq_fila)

        # Insertamos los registros de mensajes ya enviados previamente
        for i in range(5, 11):
            insertar(self.bd, 'fila',
                     *["+5493415846599",  # Telefono
                       "El mensaje jeje",  # Texto del mensaje
                       "enviado",  # Estado del mensaje
                       "Matias",  # Nombre agendado
                       "12:30 15/02/2021"])  # Fecha y hora

        pendientes = obtener_mensajes_a_enviar(self.bd)

        self.assertIsInstance(pendientes, list)
        self.assertTrue(pendientes == [])

    def test_obtener_mensajes_a_enviar_tabla_no_existe(self):
        """
        Verifica que la funcion obtener_mensajes_a_enviar levanta una excepcion cuando se la llama
        sin ninguna tabla 'fila' creada.
        """
        with self.assertRaises(sqlite3.OperationalError):
            obtener_mensajes_a_enviar(self.bd)

    def test_obtener_ultimos_mensajes_usando_telefono_sin_registros(self):
        """
        Verifica que la funcion obtener_id_ultimo_mensaje devuelve una lista vacia cuando no
        hay registros en la tabla 'mensajes'
        """
        crear_tabla(self.bd, 'mensajes', **esq_mensajes)

        ultimo_mensaje = obtener_id_ultimo_mensaje(self.bd, "+5493415846599")

        self.assertIsInstance(ultimo_mensaje, list)
        self.assertTrue(ultimo_mensaje == [])

    def test_obtener_ultimos_mensajes_usando_telefono_con_registros(self):
        """
        Verifica que la funcion obtener_id_ultimo_mensaje devuelve una lista de elementos Row
        cuando hay registros que coinciden en la tabla 'mensajes'
        """
        crear_tabla(self.bd, 'mensajes', **esq_mensajes)

        # Ingresamos los registros con el numero "+5493415846599"
        for i in range(5):
            insertar(self.bd, 'mensajes',
                     *[str(i) + "ABCD",  # Id del mensaje
                       "+5493415846599",  # Telefono
                       "Matias",  # Nombre agendado
                       "El mensaje jaja",  # Texto del mensaje
                       "txt",  # Clase del mensaje
                       "12:30 15/02/2021",  # Fecha y hora
                       "pendiente"])  # Estado del mensaje

        ultimo_mensaje = obtener_id_ultimo_mensaje(self.bd, "+5493415846599")

        self.assertIsInstance(ultimo_mensaje, list)
        self.assertTrue(ultimo_mensaje != [])

        for mensaje in ultimo_mensaje:
            self.assertIsInstance(mensaje, sqlite3.Row)
            self.assertIn('id_mensaje', mensaje.keys())

    def test_obtener_id_ultimo_mensaje_usando_telefono_sin_tabla(self):
        """
        Verifica que la funcion obtener_id_ultimo_mensaje levanta una excepcion si es llamada
        y en la base de datos no existe la tabla 'mensajes'
        """
        with self.assertRaises(sqlite3.OperationalError):
            obtener_id_ultimo_mensaje(self.bd, "+5493415846599")

    def test_obtener_usuario_que_no_existe(self):
        """
        Verifica que la funcion obtener_usuario devuelve None cuando no existe el usuario con el numero
        de telefono dado en la tabla 'usuarios'
        """
        crear_tabla(self.bd, 'usuarios', **esq_usuarios)

        usuario = obtener_usuario(self.bd, "+5493415846599")

        self.assertIsNone(usuario)

    def test_obtener_usuario_que_existe(self):
        """
        Verifica que la funcion obtener_usuario devuelve los datos del usuario dado cuando el mismo
        existe en la tabla 'usuarios'
        """
        crear_tabla(self.bd, 'usuarios', **esq_usuarios)

        insertar(self.bd, 'usuarios', *[
            '+5493415846599',
            f'Matias'
        ])

        usuario = obtener_usuario(self.bd, '+5493415846599')

        self.assertIsInstance(usuario, sqlite3.Row)
        self.assertIn('telefono', usuario.keys())
        self.assertIn('nombre_agendado', usuario.keys())
        self.assertTrue(usuario['telefono'] == "+5493415846599")
        self.assertTrue(usuario['nombre_agendado'] == "Matias")

    def test_actualizar_mensaje_en_fila_existe(self):
        """
        Verifica que un mensaje se actualizo correctamente cuando este mismo existe en la tabla 'fila'
        """
        crear_tabla(self.bd, 'fila', **esq_fila)

        insertar(self.bd, 'fila',
                 *["+5493415846599",  # Telefono
                   "El mensaje jeje",  # Texto del mensaje
                   "pendiente",  # Estado del mensaje
                   "Matias",  # Nombre agendado
                   "12:30 15/02/2021"])  # Fecha y hora

        # Usamos la id '1' ya que solo existe un registro en la tabla
        actualizar_mensaje_fila(self.bd, 1, "enviado")

        mensaje = obtener_todos(self.bd, "fila")[0]  # Obtenemos el unico registro de la tabla

        self.assertIn('estado', mensaje.keys())
        self.assertTrue(mensaje['estado'] == 'enviado')

    def test_actualizar_mensaje_en_fila_no_existe(self):
        """
        Verifica que al intentar actualizar el estado de un mensaje en la fila y este no exista,
        no hace nada.
        """
        crear_tabla(self.bd, 'fila', **esq_fila)

        insertar(self.bd, 'fila',
                 *["+5493415846599",  # Telefono
                   "El mensaje jeje",  # Texto del mensaje
                   "pendiente",  # Estado del mensaje
                   "Matias",  # Nombre agendado
                   "12:30 15/02/2021"])  # Fecha y hora

        # Intentamos actualizar el mensaje con id=2 (no existe)
        actualizar_mensaje_fila(self.bd, 2, 'enviado')

        mensaje = [msj for msj in obtener_todos(self.bd, 'fila') if msj['rowid'] == 2]

        self.assertIsInstance(mensaje, list)
        self.assertTrue(mensaje == [])

    def test_actualizar_contacto_existe(self):
        """
        Verifica que la funcion actualizar_contacto funciona correctamente cuando el usuario dado
        existe en la tabla 'usuarios'.
        """
        crear_tabla(self.bd, 'usuarios', **esq_usuarios)

        insertar(self.bd, 'usuarios', *[
            '+5493415846599',
            f'Matias'
        ])

        actualizar_contacto(self.bd, "+5493415846599", "Nuevo nombre")

        usuario = [usr for usr in obtener_todos(self.bd, 'usuarios') if usr['telefono'] == "+5493415846599"][0]

        self.assertIsInstance(usuario, sqlite3.Row)
        self.assertIn('telefono', usuario.keys())
        self.assertTrue(usuario['telefono'] == '+5493415846599')
        self.assertTrue(usuario['nombre_agendado'] == "Nuevo nombre")

    def test_actualizar_contacto_no_existe(self):
        """
        Verifica que la funcion actualizar_contacto no hace nada si el usuario dado no existe en la
        base de datos.
        """
        crear_tabla(self.bd, 'usuarios', **esq_usuarios)

        actualizar_contacto(self.bd, "+5493415846599", "Nuevo nombre")

        # Buscamos el usuario (que no existe)
        usuario = [usr for usr in obtener_todos(self.bd, 'usuarios') if usr['telefono'] == "+5493415846599"]

        self.assertTrue(usuario == [])

    def test_actualizar_estado_de_mensaje_que_existe(self):
        """
        Verifica que la funcion actualizar_estado_mensaje funcion correctamente actualizando el estado
        de un mensaje que si existe en la base de datos.
        """
        crear_tabla(self.bd, 'mensajes', **esq_mensajes)

        # Insertamos un registro con el estado en pendiente
        insertar(self.bd, 'mensajes',
                 *["ABCD",  # Id del mensaje
                   "+5493415846599",  # Telefono
                   "Matias",  # Nombre agendado
                   "El mensaje jaja",  # Texto del mensaje
                   "txt",  # Clase del mensaje
                   "12:30 15/02/2021",  # Fecha y hora
                   "pendiente"])  # Estado del mensaje

        actualizar_estado_mensaje(self.bd, "ABCD", "enviado")

        # Obtenemos el mensaje correspondiente
        mensaje = [msj for msj in obtener_todos(self.bd, 'mensajes') if msj['id_mensaje'] == "ABCD"][0]

        self.assertIsInstance(mensaje, sqlite3.Row)
        self.assertTrue(mensaje['estado'] == "enviado")

    def test_actualizar_estado_de_mensaje_que_no_existe(self):
        """
        Verifica que la funcion actualizar_estado_mensaje no hace nada si el mensaje dado no existe en la
        base de datos.
        """
        crear_tabla(self.bd, 'mensajes', **esq_mensajes)

        actualizar_estado_mensaje(self.bd, "ABCD", "enviado")

        mensaje = [msj for msj in obtener_todos(self.bd, 'mensajes') if msj['id_mensaje'] == "ABCD"]

        self.assertTrue(mensaje == [])
